
package vt.cs.db;

import java.util.*;

/**
 * A DatabaseSet is the "top-level-thing" that knows how a specific
 * type of database should be interpreted. It is responsible for 
 * providing access to the appropriate kinds of Database objects.
 * Currently, the appropriate type of this object will be instantiated
 * by a static method in the Manager class.
 * <I>Need to address creation of new database sets. Should the init()
 * method be responsible for this?</I>
 */
public abstract class DatabaseSet extends AttributeObject {
	public 		static final String PREFIX	= "DatabaseSet";

	protected	Hashtable	databases	= null;

	public DatabaseSet() {
		databases = new Hashtable();
		attributes = new Hashtable();
	}

	/**
	 * Initializes the database set based on a user-provided string.
	 * Initialization should be done here rather than in the constructor,
	 * since the Manager will instantiate the interpreter using the
	 * parameterless constructor.
	 */
	public abstract void init(String data) throws DatabaseException; 
	

	/**
	 * Returns an array of all Databases currently available.
	 * <I>Return a Vector or Enumeration instead?</I>
	 */
	public String[] getDatabases() {
		String[] 	dbs = new String[databases.size()];
		int			i	= 0;
		for (Enumeration e = databases.keys(); e.hasMoreElements();) {
			dbs[i++] = (String)e.nextElement();
		}
		return dbs;
	}

	/**
	 * Adds a database to the sets database table.
	 */
	protected void add(String name, Database db) {
		databases.put(name, db);
	}

	/**
	 * Adds a new Database. In read-only environments (eg, if 
 	 * we are accessing data via http from a remote server) we probably
	 * want to throw an exception if this is attempted. We probably also
	 * want to provide some way for the user to find out if this is read-only.
	 */
	public abstract Database newDatabase(String name);

	/**
	 * Adds the given database to this database set. Need to determine
	 * the semantics of this. Do we copy the database or just copy
	 * a ref? Will it depend on the database type? Could one implementation
	 * be handed a database from anther implementation?
	 */
	public abstract void addDatabase(Database db);

	/**
	 * Returns a ref the Database with the given name. 
	 */
	public Database	getDatabase(String name) {
		return (Database)databases.get(name);
	}

	/**
	 * Removes a Database. As with newDatabase, we need to signal
	 * failure in read-only environments. Specific implementations
	 * should probably override this method to dispose of the 
	 * database itself if appropriate.
	 */
	public void	deleteDatabase(String name) {
		databases.remove(name);
	}

}

