
package vt.cs.db.rmidb;

import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import vt.cs.db.*;

public class RemoteTableImpl extends UnicastRemoteObject implements 
	RemoteTable {

	private String 		name = null;
	private String		source = null;
	private Vector		columns = null;
	private Hashtable	columnNames = null;
	private Hashtable	attributes = null;

	RemoteTableImpl(String name, String[] columnNames, String[] columnTypes,
		String[] columnDescriptions, String source) throws RemoteException {
		super();
		int i;

		this.name = name;
		this.source = source;
		this.columnNames = new Hashtable();
		columns = new Vector();
		attributes = new Hashtable();

		for (i = 0; i < columnNames.length; i++) {
			columns.addElement( new RMIColumn(columnNames[i], columnTypes[i], 
				columnDescriptions[i], 0));
			this.columnNames.put(columnNames[i], new Integer(i));
		}
	}

	public String getName() throws RemoteException {
		return name;
	}

	public void setName(String name) throws RemoteException {
		this.name = name;
	}

	public void setAttribute(String key, Object value) throws RemoteException {
		attributes.put(key, value);
	}
	public String removeAttribute(String key) throws RemoteException {
		return (String)attributes.remove(key);
	}
	public Object getAttribute(String key) throws RemoteException {
		return attributes.get(key);
	}
	public Vector attributeKeys() throws RemoteException {
		Vector		keys	= new Vector();
		
		for(Enumeration e = attributes.keys(); e.hasMoreElements();) {
			keys.addElement(e.nextElement());
		}
		return keys;
	}

	public long rowcount() throws RemoteException {
		if (columns.size() == 0)
			return 0;
		else 
			return ((RMIColumn)(columns.elementAt(0))).data.size();
	}

	public Vector columnNames() throws RemoteException {
		int			i		= 0;
		RMIColumn	col		= null;
		Vector		names	= new Vector();
		
		for(i = 0; i < columns.size(); i++) {
			col = ((RMIColumn)(columns.elementAt(i)));
			names.addElement(col.name);
		}
		return names;
	}

	public Vector columnNames(String type) throws RemoteException {
		int			i		= 0;
		RMIColumn	col		= null;
		Vector		names	= new Vector();
		
		for(i = 0; i < columns.size(); i++) {
			col = ((RMIColumn)(columns.elementAt(i)));
			if (col.type.equals(type))
				names.addElement(col.name);
		}
		return names;
	}

	public String description(String columnName) throws RemoteException {
		Integer	columnNumber =  null;

		columnNumber = (Integer) columnNames.get(columnName);
		return ((RMIColumn)
			(columns.elementAt(columnNumber.intValue()))).description;
	}

	public Object get(int rowNumber, int columnNumber) throws RemoteException {
		return ((RMIColumn)(columns.elementAt(columnNumber))).data.elementAt(
			rowNumber);
	}

	public Object get(int rowNumber, String columnName) throws RemoteException {
		Integer	columnNumber =  null;
		columnNumber = (Integer) columnNames.get(columnName);
		return get(rowNumber, columnNumber.intValue());
	}

	public Vector getColumn(String columnName) throws RemoteException {
		Integer	columnNumber =  null;
		columnNumber = (Integer) columnNames.get(columnName);
		return getColumn(columnNumber.intValue());
	}

    public Vector getColumn(int columnNumber) throws RemoteException {
		return (Vector) (((RMIColumn)(columns.elementAt(
			columnNumber))).data.clone());
	}

	public void appendRow(Object[] rowData) throws RemoteException {
		int i = 0;
		if (rowData.length != columns.size()) {
			/* Throw an exception here */
		}
		else {
			for (i = 0; i < rowData.length; i++) {
				/* Need to verify type here... */
				((RMIColumn)(columns.elementAt(i))).data.addElement(rowData[i]);
			}
		}
	}

}

class RMIColumn {
	public	int		startsAt = 0;
	public	String	name = null;
	public	String	type = null;
	public 	String	description = null;
	public	Vector	data = null;

	public RMIColumn(String name, String type, String description,
		int startsAt) {
		this.name = name;
		this.type = type;
		this.description = description;
		this.startsAt = startsAt;
		data = new Vector();
	}

}



