/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/shareablePrimitive/ShareableInteger.java,v 1.1 1997/05/16 22:29:46 begolej Exp $
 *
 */

package vt.cs.shareablePrimitive;

/**
 *
 * @author	James "Bo" Begole
 *
 * @version $Revision $
 * @see     vt.cs.shareablePrimitive.Byte
 * @see     vt.cs.shareablePrimitive.Double
 * @see     vt.cs.shareablePrimitive.Float
 * @see     vt.cs.shareablePrimitive.Integer
 * @see     vt.cs.shareablePrimitive.Long
 * @see     vt.cs.shareablePrimitive.Short
 */
 
import SharedData.*;
import SharedData.primitive.*;
 
public class ShareableInteger extends java.lang.Number {
    /**
     * The smallest value of type <code>int</code>. 
     *
     * @since   JDK1.0
     */
    public static final int   MIN_VALUE = 0x80000000;

    /**
     * The largest value of type <code>int</code>. 
     *
     * @since   JDK1.0
     */
    public static final int   MAX_VALUE = 0x7fffffff;

    /**
     * The session if this value is shared.  When session == null
     * this value is not shared.
     */
    
    java.lang.Integer local = null;
    SharedData.primitive.SharedInteger shared = null;
    
    public ShareableInteger(String name, Session session, Client client, int value) {    
			       
    	if ((session == null) || (name == null) || (client == null)) {
    	    local = new java.lang.Integer(value);
            debug("constructor: " +
                  " name: "         + name +
                  " session name: " + session +
                  " client name: "  + client +
                  " value: " + value);
    	} else {
    	    shared = new SharedData.primitive.SharedInteger(name, session, client, value);
            debug("constructor: " +
                  " name: "         + name +
                  " session name: " + session.getName() +
                  " client name: "  + client.getName() +
                  " value: " + value);
        }
    	    
    }
    
    public ShareableInteger(String name, Session session, Client client, String s) {
        this(name, session, client, java.lang.Integer.parseInt(s));    
    }
    
    public ShareableInteger(String s) {	
        this(null, null, null, s);    	
    }
    
    public ShareableInteger(int value) {    
    	this(null, null, null, value);
    }

    /**
     * Returns the value of the specified number as an <code>int</code>.
     * This may involve rounding.
     *
     * @return  the numeric value represented by this object after conversion
     *          to type <code>int</code>.
     * @since   JDK1.0
     */
    public int intValue() {
        if (local !=null)
            return local.intValue();
        else
            return shared.intValue();
    }

    /**
     * Returns the value of the specified number as a <code>long</code>.
     * This may involve rounding.
     *
     * @return  the numeric value represented by this object after conversion
     *          to type <code>long</code>.
     * @since   JDK1.0
     */
    public long longValue() {
        if (local !=null)
            return local.longValue();
        else
            return shared.longValue();
    }

    /**
     * Returns the value of the specified number as a <code>float</code>.
     * This may involve rounding.
     *
     * @return  the numeric value represented by this object after conversion
     *          to type <code>float</code>.
     * @since   JDK1.0
     */
    public float floatValue() {
        if (local !=null)
            return local.floatValue();
        else
            return shared.floatValue();
    }
    
    /**
     * Returns the value of the specified number as a <code>double</code>.
     * This may involve rounding.
     *
     * @return  the numeric value represented by this object after conversion
     *          to type <code>double</code>.
     * @since   JDK1.0
     */
    public double doubleValue() {
        if (local !=null)
            return local.doubleValue();
        else
            return shared.doubleValue();
    }

    public String toString() {
        if (local !=null)
            return local.toString();
        else
            return shared.toString();
    }    


  private static final boolean debug = true;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}
