package vt.cs.virtSchool.MinimalServer;

import vt.cs.db.rmidb.*;
import SharedData.*;

public class MinimalServer {
  public static final String NAME = "MinimalServer";

  static Server serverStub = null;
  static ServerImpl dbServer = null;


  public static final int  defaultPort = 4445;
  public static final String defaultHost = "simon.cs.vt.edu";
  public static final String defaultSessionName = "BnPSession1";
  public static final String defaultChannelName = 
    vt.cs.virtSchool.labapplets.BlockAndPlane.SimulationControl.ChannelName;

  private static String sessionName = defaultSessionName;

  public synchronized static void main(String args[]) {
    startDBServer();
    startSharedDataServer(args);
    System.out.println("MinimalServer started...");
  }

  private static void startDBServer() {
    System.setSecurityManager(new java.rmi.RMISecurityManager());
    // set up the database server
    try {
      serverStub = (Server)java.rmi.Naming.lookup(Server.NAME);
    }
    catch (Exception e) {
      try {
	dbServer = new ServerImpl(Server.NAME);
	java.rmi.Naming.bind(Server.NAME, dbServer);
	serverStub = (Server)java.rmi.Naming.lookup(Server.NAME);
      }
      catch (Exception ex) {
	ex.printStackTrace();
	System.exit(1);
      }
    }
  }


  private static void startSharedDataServer(String args[]) {
    debug("startSharedDataServer");

    // Set up the SharedData server
    Session    session = null;
    String     url         = null;
    String     sessionType = null;
    String     hostname    = null;
    String     channelName = null;
    int        hostport    = 0;


    hostname    = getHost(args);
    hostport    = getPort(args);
    sessionType = getType(args);
    channelName = getChannel(args);
    
    url = "coll://" + hostname + ":" + hostport + "/" + sessionType +
      "/Session/" + sessionName; 
	      
    try {
      /* Create a session, [re]bind it and create a channel. */
      session = SessionFactory.createSession(url);
      //      session.createChannel(channelName, true);
      System.out.println("Setup and bound server.");
    } catch (SharedDataException e) {
      System.out.println("MinimalServer.startSharedDataServer():"+
			 " shared data exception: " + e);
      e.printStackTrace();
    }
  }


  private static String getHost(String args[]) {
    debug("getHost()");
    String returnString = getArg("-server", args);
    if (returnString != null)
      return returnString;
    else
      return defaultHost;
  }

  private static String getChannel(String args[]) {
    debug("getChannel()");
    String returnString = getArg("-channel", args);
    if (returnString != null)
      return returnString;
    else
      return defaultChannelName;
  }

  private static int getPort(String args[]) {
    debug("getPort");
    String returnString = getArg("-port", args);
    if (returnString != null)
      return(Integer.parseInt(returnString));
    else
      return defaultPort;
  }


  private static String getType(String args[]) {
    debug("getType.");
    String returnString = getArg("-type", args);
    if (returnString != null)
      return returnString;
    else
      return "socket";
  }


  private static String getArg(String parameter, String args[]) {
    debug("getArg.");
    int length = args.length;
    for (int i = 0; i < length; i++) {
      if (args[i].equals(parameter)) {
	if (++i < length) {
	  return(args[i]);
	}
      }    
    }
    return null;
  }

  private static  final boolean debug = true;
  private static void debug(String msg) {
    if (debug)
      System.out.println("sieve.crew.Server: "+msg);
  }    
}
