/*
 *
 * Written By:   Philip Isenhour
 * Date:         25 September 1996
 * Description:  Prototype Table view class. The table contains instantiated
 *				 activities.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/TableView.java,v 1.7 1997/05/15 17:31:23 begolej Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.client;

import netscape.application.*;
import netscape.util.*;
import java.util.*;
import java.rmi.*;
import vt.cs.virtSchool.server.*;
import vt.cs.db.*;
import SharedData.*;

/**
 * A TableView contains icons for instantiated activities which may be
 * double-clicked to run the activity applet. Among other minor glitches,
 * the handling of links could use some re-design work.
 *
 * @version $Id: TableView.java,v 1.7 1997/05/15 17:31:23 begolej Exp $
 */
public class TableView extends View implements DragDestination,
  ActivityContainer {
  private java.util.Hashtable activities = null;
  private java.util.Hashtable links = null;
  private Client client = null;
  private Channel channel = null;
  private ActivityChannelConsumer consumer = null;

  private Point linkStartPoint = null;
  private Point linkEndPoint = null;

  private String databaseSetRoot = null;
  private String sessionURLbase = null;
  private RoomData room = null;

  public TableView(Client client, Channel channel, RoomData room,
		   int x, int y, int width, int height) {
    super(x, y, width, height);
    this.room = room;
    this.client = client;
    this.channel = channel;
    this.consumer = new ActivityChannelConsumer(client, channel, this);
    try {
      sessionURLbase = room.getSessionURLbase();
      databaseSetRoot = room.getDatabaseSetName() + 
	"/" + DatabaseSet.PREFIX + "/";
    } catch (RemoteException rex) {
      System.out.println("TableView Error getting room information: "+rex);
      rex.printStackTrace();
    }
    activities = new java.util.Hashtable();
    links = new java.util.Hashtable();

    setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
    setBuffered(true);		
  }

  public void drawView(Graphics g) {
    g.setColor(Color.white);
    g.fillRect(0, 0, width(), height());

    //Bitmap.bitmapNamed("wmarble.gif").drawTiled(g, bounds());

    g.setColor(Color.black);
    g.drawRect(0, 0, width(), height());

    if (linkStartPoint != null) {
      g.setColor(Color.lightGray);
      g.drawLine(linkStartPoint.x, linkStartPoint.y,
		 linkEndPoint.x, linkEndPoint.y);
    }

    g.setColor(Color.darkGray);
    for (netscape.util.Enumeration e = subviews().elements(); 
	 e.hasMoreElements();) {
      View tmp = (View)e.nextElement();
      if (tmp instanceof LinkView) {
	Point p1 = ((LinkView)tmp).act1Center();
	Point p2 = ((LinkView)tmp).act2Center();
	g.drawLine(p1.x, p1.y, p2.x, p2.y);
	((LinkView)tmp).moveTo((p1.x + p2.x - LinkView.HANDLE_SIZE)/2,
			       (p1.y + p2.y - LinkView.HANDLE_SIZE)/2);
      }
    }
  }

  public boolean dragDropped(DragSession dragSession) {
    ActivityView	av 			= null;
    Rect			dragBounds	= null;

    if (dragSession.source() instanceof ActivityView) {
      av = (ActivityView) dragSession.source();
      dragBounds = dragSession.destinationBounds();

      /*
       * If the activity icon came from somewhere other than
       * the table, instantiate a new activity on the table.
       */
      if (! av.descendsFrom(this)) {
	Activity act = new Activity(av.act, client.getName());
	act.setSessionURLbase(sessionURLbase);
	act.x = dragBounds.x;
	act.y = dragBounds.y;

	try {
	  // have room  create and hold open the session for this activity
	  room.holdSession(act.getSessionURL());
	} catch (RemoteException rex) {
	  System.out.println("Error creating activity session: "+
			     act.getSessionURL());
	  rex.printStackTrace();
	}

	activityAdded(act);
	/*
	 * Create a new database for this activity and send a 
	 * message to the other clients
	 */
	try {
	  String databaseSetName =
	    databaseSetRoot + act.databaseId();
	  DatabaseSet dbSet = Manager.getDatabaseSet(databaseSetName);
	  debug("databaseSetName = "+databaseSetName);

	  act.addURLParameter("DatabaseSet", databaseSetName);
	  if (act.generatesData()) {
	    Database db = dbSet.newDatabase(act.databaseId());
	    act.addURLParameter("Database", act.databaseId());
	  }
				
	  channel.uniformSendToOthers(client, new Data(Message.toByteArray(
									   new Message(Message.ACTIVITY_ADD,
										       act))));
	}
	catch(Exception e) {
	  e.printStackTrace();
	}
      }
      else {
	av.moveActivityTo(dragBounds.x, dragBounds.y);
	try {
	  channel.uniformSendToOthers(client, new Data(Message.toByteArray(
									   new Message(Message.ACTIVITY_MOVE, av.act))));
	}
	catch(Exception e) {
	  e.printStackTrace();
	}
      }
      draw();
      return true;
    }
    return false;
  }

  public DragDestination acceptsDrag(DragSession dragSession,
				     int x, int y) {
    /* Check type first? */
    return this;
  }

  public void makeLink(ActivityView actView1, ActivityView actView2) {
    Link	link = null;

    /*
     * Don't allow links between two consumers. Eventually we will
     * probabily want to make this a bit more sophisticated.
     */
    if (!actView1.act.generatesData() && !actView2.act.generatesData())
      return;

    /*
      And don't allow links between two only producers
     */
    if (!actView1.act.readsData() && !actView2.act.readsData())
      return;

    link = new Link(actView1.act.instantiatedBy, actView1.id(),
		    actView2.id());
    /*
     * Add each activity's database to the other activity's 
     * database set.
     */
    try {
      DatabaseSet dbSet1 = Manager.getDatabaseSet(
						  databaseSetRoot + actView1.act.databaseId());

      DatabaseSet dbSet2 = Manager.getDatabaseSet(
						  databaseSetRoot + actView2.act.databaseId());

      if (actView1.act.generatesData()) {
	dbSet2.addDatabase(
			   dbSet1.getDatabase(actView1.act.databaseId()));
      }
      if (actView2.act.generatesData()) {
	dbSet1.addDatabase(
			   dbSet2.getDatabase(actView2.act.databaseId()));
      }

      linkAdded(link);

      channel.uniformSendToOthers(client,
				  new Data(Message.toByteArray(new Message(Message.LINK_ADD,
									   link))));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void removeLink(Link link) {
    ActivityView	actView1 = 
      (ActivityView) activities.get(link.activity1ID);

    ActivityView	actView2 = 
      (ActivityView) activities.get(link.activity2ID);
    /*
     * Remove databases from database sets.
     */
    try {
      DatabaseSet dbSet1 = Manager.getDatabaseSet(
						  databaseSetRoot + actView1.act.databaseId());

      DatabaseSet dbSet2 = Manager.getDatabaseSet(
						  databaseSetRoot + actView2.act.databaseId());

      if (actView1.act.generatesData()) {
	dbSet2.deleteDatabase(actView1.act.databaseId());
      }
      if (actView2.act.generatesData()) {
	dbSet1.deleteDatabase(actView2.act.databaseId());
      }

      linkRemoved(link);	

      channel.uniformSendToOthers(client,
				  new Data(Message.toByteArray(new Message(Message.LINK_REMOVE,
									   link))));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void linkFinished() {
    linkStartPoint = linkEndPoint = null;
    draw();
  }

  public boolean dragEntered(DragSession dragSession) {
    return true;
  }
 
  public boolean dragMoved(DragSession dragSession) {
    return true;
  }
 
  public void dragExited(DragSession dragSession) {
  }

  public void drawLink(ActivityView activity, int x, int y) {
    linkEndPoint = activity.convertPointToView(this, new Point(x, y));
    linkStartPoint = new Point(activity.bounds().midX(), 
			       activity.bounds().midY());
    draw();
  }

  public void moveLink(LinkView link, ActivityView act1, ActivityView act2) {
    Point p1 = link.act1Center();
    Point p2 = link.act2Center();
    link.moveTo( (p1.x + p2.x - LinkView.HANDLE_SIZE)/2, 
		 (p1.y + p2.y - LinkView.HANDLE_SIZE)/2);
  }

    /**
     * Uses the <CODE>Environment class to open an activity applet. 
     * We originally restricted data-producing applets to a single
     * executing instance, but that functionality was removed to accomodate
     * other structural changes. It should probably be re-implemented
     * (in a less restrictive way, to accomodate truly collaborative apps)
     * using the SharedData.Token class.
     */
  public void openActivity(ActivityView actView) {
    ActivityFrame	frame = actView.getFrame();
    if (frame != null)
      frame.toFront();
    else {
      /*
       * For now we will only ask permission to launch
       * activity applets which produces data. Eventually
       * we will need to re-think this as true collaborative
       * applets are introduced into the system.

       * NEED to use SharedData.Token here...
       try {
       if ((!act.generatesData()) || 
       room.permissionToLaunchActivity(act, user))
       */
		
      actView.setFrame(Environment.execute(actView.act, this));
      /*
	else
	System.out.println("Permission Denied");
	}
	catch(Exception e) {
	e.printStackTrace();
	}
	*/
    }
  }

  public void closeActivity(Activity act) {
    ActivityView actView = (ActivityView)activities.get(act.id());
    actView.setFrame(null);
  }

    /**
     * This method needs to be implemented. At a minimum, it needs to
     * check for active ActivityFrames and kill those off so that
     * they aren't hanging around after the user leaves a room.
     */
  public void shutdown() {
    /*
     * Leave the channel (?)
     */

    /*
     * Close any open activity frames.
     */
  }

    /* ---------- ActivityContainer Interface Methods ---------- */
  public void activityAdded(Activity act){
    ActivityView actView    = null;
    actView = new ActivityView(act, client.getName());
    activities.put(act.id(), actView);
    addSubview(actView);
    draw();
  }
  public void activityMoved(Activity act){
    ActivityView actView    = null;
    actView = (ActivityView) activities.get(act.id());
    if (actView != null) {
      actView.update(act);
      draw();
    }
  }
  public void activityRemoved(Activity act){}
 
  public void activityClaimed(Activity act){}
  public void activityReleased(Activity act){}
  public void linkAdded(Link link) {
    ActivityView    act1        = null;
    ActivityView    act2        = null;
    LinkView        linkView    = null;
    Point           p1          = null;
    Point           p2          = null;
 
    act1 = (ActivityView) activities.get(link.activity1ID);
    if (act1 == null) {
      System.out.println("No activity (1) with ID: " + link.activity1ID);
      return;
    }
    act2 = (ActivityView) activities.get(link.activity2ID);
    if (act1 == null) {
      System.out.println("No activity (2) with ID: " + link.activity2ID);
      return;
    }
    linkView = new LinkView(link, act1, act2);
 
    p1 = linkView.act1Center();
    p2 = linkView.act2Center();
 
    addSubview(linkView);
    linkView.moveTo((p1.x + p2.x - LinkView.HANDLE_SIZE)/2,
		    (p1.y + p2.y - LinkView.HANDLE_SIZE)/2);
    draw();

    links.put(link.id(), linkView);
  }

  public void linkRemoved(Link link) {
    removeSubview((LinkView)(links.get(link.id())));
    links.remove(link.id());
    draw();
  }

  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }
 
}

