/*
 *
 * Written By:   Philip Isenhour
 * Date:         3 December 1996
 * Description:  Class for displaying a list of users currently joined
 *               to a session.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/world/RoomApplet.java,v
 1.5 1996/11/20 15:31:31 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.client;
import netscape.application.*;
import SharedData.*;
import vt.cs.virtSchool.server.*;


/**
 * A <CODE>UserList</CODE> observes a session and displays a list of
 * current users (clients) that have joined the session.
 * Private chat sessions (whenever they get implemented) could be launched
 * from here or from a subclass of <CODE>UserList</CODE>.
 *
 * @version $Id: UserList.java,v 1.4 1997/02/19 19:43:38 begolej Exp $
 */
public class UserList extends ListView implements Target, SessionObserver {
    private Session session = null;
 
    public UserList(int x, int y, int w, int h) {
        super(x, y, w, h);
        setPrototypeItem(new UserListItem());
		prototypeItem().setTitle("User Name");
        setAllowsEmptySelection(true);
        setAllowsMultipleSelection(false);

        setCommand("click");
        setDoubleCommand("doubleclick");
        setTarget(this);
        //setTransparent(false);
        setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
        //setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);
		sizeToMinSize();
    }

    public void update(Session session) {
        String  clients[]   = null;
        int     i           = 0;

        removeAllItems();
		sizeToMinSize();
		draw();
 
        try {
            if (this.session != null)
                this.session.deleteObserver(this);
 
            this.session = session;
            session.addObserver(this);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        try {
            clients = session.listClientNames();
            if (clients != null) {
                for (i = 0; i < clients.length; i++) {
                    addItem(clients[i]);
                }
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        draw();
    }
 
    public void addItem(String name) {
		/*
		 * Currently the server's "client" component (in RoomDataImpl)
		 * has a name which starts with an "_".
		 */
        if (! name.startsWith("_")) {
            ListItem    item    = addItem();
            item.setTitle(name.substring(0, name.lastIndexOf('.')));
			sizeToMinSize();

	        Rect r = rectForItem(item);
    	    /*
			 * Set the width to a small number to insure we're 
			 * scrolled hard left
			 */
       		r.width = 10;
        	scrollRectToVisible(r);
        	draw();
        }
    }
 
    public void performCommand(String command, Object object) {
        if (command.equals("click")) {
            //update(session);
        }
        else if (command.equals("doubleclick")) {
            System.out.println("Double-Clicked");
        }
    }
 
    /* ---------- SessionObserver Interface methods ---------- */
    public void sessionChanged(SessionEvent event) {
        update(event.getSession());
    }
    public String getName() {
        return null;
    }
    /* ------------------------------------------------------- */
 
}

class UserListItem extends ListItem {

	public UserListItem() {
	}

	public int minHeight() {
		return 20;
	}

	public int minWidth() {
		return 100;
	}

	protected void drawBackground(Graphics g, Rect boundsRect) {
        if (!isSelected()) {
            g.setColor(listView().backgroundColor());
            g.fillRect(boundsRect);
        }
		else {
            g.setColor(Color.white);
            g.fillRect(boundsRect);
		}
	}
}

