/*
 *
 * Written By:   James "Bo" Begole
 * Date:         April 23, 1997
 * Description:  Consumer for a channel that recieves block and plane control 
 *                   event messages.
 *                   These messages will be converted to calls to a
 *                   BlockAndPlane object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 begolej Exp $
 *
 * Copyright (c) 1996 James Bo Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.labapplets.BlockAndPlane; 

import sieve.crew.*;
import SharedData.*;

/**
 * Consumer for a channel that recieves control 
 * event messages.
 * These messages will be converted to calls to a
 *  object.
 *
 * @version $Revision  $
 */

public class SimulationControl extends RemoteControl {
  public static final String ChannelName = "SimulationControl";

  private BlockAndPlane bnp = null;

  public SimulationControl(Session session, BlockAndPlane bnp) {
    super(session, ChannelName);
    this.bnp = bnp;
    debug("Construction Complete"); 
  }

  /* ------- ChannelConsumer Interface methods ------ */
  public void dataReceived(Data data) {
    Message msg = null;
    msg = (Message)Message.messageFromData(data);
    switch(msg.type) {
    case BnPMessage.RUN:
      bnp.runSimulation(false);
      break;
    case BnPMessage.PAUSE:
      bnp.pauseSimulation();
      break;
    case BnPMessage.RESET:
      bnp.resetSimulation();
      break;
    default:
      System.err.println("Unknown message type received.  Ignoring. "+msg.type);
    }
  }


  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  } 
}
