/*
 *
 * Written By: Philip Isenhour
 * Date:       21 August 1996
 * Descriptions: Simple simulation of inelastic collision
 *
 */

/* 
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/labapplets/Collision/Collision.java,v 1.1 1996/09/17 06:46:05 isenhour Exp $
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.labapplets.Collision;

import vt.cs.virtSchool.objectManager.ObjectManager;
import vt.cs.virtSchool.util.*;
import vt.cs.widgets.MultiColumn;
 
import java.rmi.Remote;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.Naming;
 
import java.applet.*;
import java.awt.*;
import java.net.*;
import java.util.*;

public class Collision extends Applet implements Runnable {

	private TextField		title;
	private Thread			thd;

	private Dimension 		d;
	private Car 			car1, car2;

	private static final String logDataKey = "Collision Log";
	private ProducerStream		logStream;

	public void init() {

		logStream = new ProducerStream(this, "Collision Log", 
			"java.lang.String");
		logStream.write(new String("\nStarted at " + new Date().toString()));

		d = size();

		setLayout(new BorderLayout());
		title = new TextField("Inelastic Collision Simulation");
		title.setEditable(false);
		add("North", title);

	  	car1 = new Car(Car.WIDTH, 2 * d.height/3 - 2, d.width, 100, 1, 
			Color.red, "Red Car", this, logStream);

	  	car2 = new Car(d.width, 2 * d.height/3 - 2, 
			d.width, -50, 10, Color.blue, "Blue Car", this, logStream);
/*
	  	car2 = new Car((d.width/2) - (Car.WIDTH/2), 2 * d.height/3 - 2, 
			d.width, -100, 1, Color.blue);
*/
		thd = new Thread(this);
		thd.setPriority(2);
		thd.start();
	}

	public void run() {
		do {
			try {
				Thread.sleep(25);
			}
			catch (InterruptedException e) {return;}
			paintCars(getGraphics());
		} while (car1.moving() || car2.moving());
	}

	public void stop() {
		if (thd != null)
			thd.stop();
	}

	public void paint(Graphics g) {
		g.setColor(Color.black);
		g.fillRect(0, 2 * d.height/3, d.width, 5);
		paintCars(g);
	}

	public void paintCars(Graphics g) {
		Image offscreen = createImage(d.width, d.height);
		if (offscreen == null)
			return;
		Graphics gBuf = offscreen.getGraphics();

		gBuf.clipRect(0, ( 2 * d.height/3) - Car.HEIGHT, d.width, Car.HEIGHT);

		car1.checkCollision(car2);

		car1.paint(gBuf);
		car2.paint(gBuf);

		g.clipRect(0, ( 2 * d.height/3) - Car.HEIGHT, d.width, Car.HEIGHT);
		g.drawImage(offscreen, 0, 0, this);
	}

	public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY)
			System.exit(0);
		return false;
	}
}

class Car {
	int 	x, y, v, mass, frameWidth, initialX;
	Color 	color;
	boolean	moving = false;
	Date	started = null;
	public final static int WIDTH = 40;
	public final static int HEIGHT = 15;
	String	name = "Unnamed Car";

	int		lastVelocity = -1;
	ProducerStream	velocities;
	ProducerStream	log;

	public Car(int initX, int initY, int frameW, int initV, int m, 
		Color c, String name, Applet app, ProducerStream logstream) {
		this.name = name;
		log = logstream;
		velocities = new ProducerStream(app, name + " Velocity", 
			"java.lang.Long");
		x = initX;
		initialX = initX;
		y = initY;
		changeVelocity(initV);
		frameWidth = frameW;
		mass = m;
		color = c;
		moving = (v != 0);
		if (moving) {
			started = new Date();
		}
	}

	void changeVelocity(int newVelocity) {
		v = newVelocity;
		if (newVelocity != lastVelocity) {
			moving = moving && (x <= frameWidth) && (x-WIDTH >= 0) && (v != 0);
			velocities.write(new Long(v));
			log.write(new String("Changing velocity of " +
					name + " to " + Integer.toString(v) +
					" at position " + Integer.toString(x) + " (" +
					new Date().toString() + ")"));
			lastVelocity = v;
		}
	}

	public void paint(Graphics g) {
		moving = moving && (x <= frameWidth) && (x-WIDTH >= 0) && (v != 0);
		if (moving) {
			Date now = new Date();
			long diff = now.getTime() - started.getTime();
			//x = initialX + (int)((v * diff)/1000);

			int	 distance = (int)((v * diff)/1000);
			if (distance > WIDTH/3)
				distance = WIDTH/3;
			started = now;
			x += distance;
		}
		else if (v != 0)
			changeVelocity(0);

		g.setColor(color);
		g.fillRect(x - WIDTH, y - HEIGHT, WIDTH, HEIGHT);
	}

	public void checkCollision(Car c) {
		int diff = x - c.x;
		if (diff < 0)
			diff = -diff;
		if (diff < Car.WIDTH) {

			if ((x < frameWidth) && (x-WIDTH >= 0) && 
				(c.x < frameWidth) && (c.x-WIDTH >= 0))  {

				log.write(new String("Collision detected " +
					"at position " + Integer.toString(x) + " (" +
					new Date().toString() + ")"));
				
				changeVelocity(((mass * v) + (c.mass * c.v)) / (mass + c.mass));
				c.changeVelocity(v);

				// Move them apart
				if (x < c.x)
					x = c.x - WIDTH;
				else if (x > c.x)
					x =  c.x + WIDTH;
			}
			else {
				changeVelocity(0);
				c.changeVelocity(0);
			}
			c.moving = moving = (v != 0);	
			started = new Date();
			initialX = x;
			c.started = started;
			c.initialX = c.x;
		}
	}

	public int velocity() {
		return v;
	}

	public boolean moving() {
		return moving;
	}

}


