/*
 *
 * Written By:   Craig Struble, 
 *               James "Bo" Begole
 * Date:         1 July 1996
 * Description:  Reaction Timer using remote object to store data
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/labapplets/ReactionTimer/Reaction.java,v 1.2 1996/09/17 18:27:13 isenhour Exp $
 *
 * Copyright (c) 1996 Craig Struble, James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */



package vt.cs.virtSchool.labapplets.ReactionTimer;

//import vt.cs.ro.remote.*;
import vt.cs.virtSchool.objectManager.*;
import vt.cs.virtSchool.util.*;

import java.rmi.Remote;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.Naming;

import java.util.*;
import java.applet.*;
import java.awt.*;
import java.net.*;

public class Reaction extends Applet {
  private final boolean debug = false;
	
  Flasher flasher;
  private ObjectManager objectManager = null;
  private DataSet dataSet = null;

  public void init() {

    // Get parameters
    RemoteObject remObjUtil = new RemoteObject();
    objectManager = remObjUtil.getObjectManager(this);
    dataSet = remObjUtil.getDataSet(this, objectManager);



    setLayout(new BorderLayout());
    add("Center", flasher = new Flasher(dataSet));
    flasher.start();
  }

}

//
// Test your reaction time with a flashing dot
//
class Flasher extends Panel implements Runnable {
  private final boolean debug = false;

  Panel buttonPanel;
  Button startButton;
  Button saveButton;
  FlashCanvas flashComponent;
  Date flashDate = null;
  Date clickDate = null;

  Thread life = null;
  private DataSet dataSet = null;

  public Flasher(DataSet dataSet) {
    this.dataSet = dataSet;
		
    setLayout(new BorderLayout());
    add("North", buttonPanel = new Panel());
    add("Center", flashComponent = new FlashCanvas());

    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add("Start", startButton = new Button("Start"));
    buttonPanel.add("Separator", new vt.cs.widgets.Separator(vt.cs.widgets.Separator.VERTICAL));
    //    buttonPanel.add("Load", new Button("Load"));
    //    buttonPanel.add("Save", saveButton = new Button("Save"));

    flashComponent.setForeground(Color.white);
    flashComponent.setBackground(Color.black);
    //    saveButton.disable();

  }



  
  public boolean action(Event evt, Object arg) {
    if ("Start".equals(arg)) {
      startButton.disable();
      flashComponent.startTicking();
    }
/*  
    else if ("Load".equals(arg)) {
      String url=null;
      try {
	url = objectManager.retrieveNamedObject("DataSet1");
	if (debug) System.out.println("DataSet URL= "+url);
      } catch (RemoteException re) {
	System.out.println("Remote Exception during retrieveNamedObject: "+re);
      }
      getServer(url);
      startButton.enable();
      saveButton.enable();
    } else if ("Save".equals(arg)) {
      try {
	objectManager.saveNamedObject("DataSet1");
      } catch (RemoteException re) {
	System.out.println("Remote Exception during saveNamedObject: "+re);
      }
    }
*/
    return true;
  }

  public boolean mouseDown(Event evt, int x, int y) {
    startButton.enable();
    if (flashComponent.newData) {
      if (dataSet == null) 
	System.out.println("No DataSet");
      if (flashComponent == null) 
	System.out.println("No FlashComponent");
      if (debug) System.out.println("Sending Reaction time: "+flashComponent.reactionTime);
      try {
	dataSet.addPoint(flashComponent.reactionTime);
      if (debug) System.out.println("Sent Reaction time: "+flashComponent.reactionTime);
      } catch (RemoteException re) {
	System.out.println("Flasher: mouseDown: "+re);
	re.printStackTrace();
      }

      flashComponent.newData = false;
    }
    return true;
  }
	

  public void start() {
    if (life == null) {
      life = new Thread(this);
      life.start();
    }
  }	
		
  public void stop() {
    //		if (debug) System.out.println("stop: ");
    if (life != null) 
      life.stop();
    life = null;
  }
	
  public void run() {}
}

//
// Canvas for drawing on
//  
class FlashCanvas extends Canvas implements Runnable {
  private final boolean debug = false;

  Random nextTime = null;
  boolean clicked = false;
  boolean printNow = false;
  long flashTime = 0;
  long reactionTime = 0;
  int displayX, displayY;
  Thread flashThread = null;

  boolean newData = false;

  public FlashCanvas() {
    nextTime = new Random(System.currentTimeMillis());
  }
  
  public boolean mouseDown(Event evt, int x, int y) {
    if (!clicked) {
      reactionTime = System.currentTimeMillis() - flashTime;
      newData = true;
      clicked = true;
      repaint();
      if (flashThread.isAlive()) {
	flashThread.stop();
      }
    }
    return false;
  }

  boolean timerStarted = false;

  public void paint(Graphics g) {
    if (clicked == true) {
      g.drawString("Your reaction time was " + reactionTime + " milliseconds.", 0 , 50);
    }
    if (printNow == true) {
      g.drawString("Click Now!", displayX, displayY);
      if (debug) System.out.println("FlashTime = "+flashTime+" \tCurrent Time = "+System.currentTimeMillis()+" before flashTime reset");
      // Reset the flashtime to the current time to make up for 
      // delays in execution up to the drawString line above
      flashTime = System.currentTimeMillis();
      if (debug) System.out.println("FlashTime = "+flashTime+" \tCurrent Time = "+System.currentTimeMillis()+" after flashTime reset");
    }
  }

  public void run() {
    while (true) {
      if (timerStarted) {
	try {
	  flashThread.sleep(flashTime - System.currentTimeMillis());
	} catch (InterruptedException e) {
	  return;
	}
	timerStarted = false;
	if (debug) System.out.println("FlashTime = "+flashTime+" \tCurrent Time = "+System.currentTimeMillis());
	printNow = true;
	paint(getGraphics());
      }
    }
  }


  public void startTicking() {
    clicked = false;
    printNow = false;
    timerStarted = true;
    repaint();

    displayX = Math.abs(nextTime.nextInt()) % (size().width - 50);
    displayY = Math.abs(nextTime.nextInt()) % (size().height - 20) + 20;
    flashTime = System.currentTimeMillis() + (Math.abs(nextTime.nextLong()) % 5000) + 2000;
    flashThread = new Thread(this);
    flashThread.start();
  }
}

