
//
//	Graphic.java
//
//	Copyright 1996,	Black Diamond Software.	 All rights reserved.
//
//	Written	by: Jason Beaver and Tyler Gingrich
//

package vt.cs.virtSchool.labapplets.WhiteBoard;

import netscape.application.*;
import netscape.util.*;

public class Graphic extends Object // implements EventFilter 
{
	Color strokeColor;
	Color fillColor;
	boolean	filled;
	boolean	selected = false;
	boolean	manipulatingGraphic;
	int anchor;

	String	ID;

	// Anchor points...
	public static final int	NONE = -1;

	public Graphic() {
		super();

		manipulatingGraphic = false;
	}

	// Called to set the receiver up for creating...
	// It should be	overridden to set the graphics inital state
	// and default anchor point.
	public void createGraphic(MouseEvent event,SimpleDrawView view)	{
		// Set the defaults to the inspector's current settings...
		strokeColor = view.currentStrokeColor();
		fillColor = view.currentFillColor();
		filled = view.currentFilledState();

		// Add ourself so that we will receive updateGraphic messages...
		view.addGraphic(this);
		prepareForManipulation(event,view);
	}

	// Called to set the receiver up for resizing...
	public void resizeGraphic(MouseEvent event,SimpleDrawView view)	{
		prepareForManipulation(event,view);
		anchor = anchorForHitHandle(event,view);
	}

	// This	is called repeatedly as	the user manipulates the graphic...
	// It should be	overridden to modify the graphic based on the
	// current anchor point	and the	current	mouse event.
	public void updateGraphic(MouseEvent event,SimpleDrawView view)	{
	}

	// Called to conclude any graphic manipulation...
	public void concludeGraphic(MouseEvent event,SimpleDrawView view) {
		finishManipulation(event,view);
//System.out.println("Finished manipulating graphic ID: " + ID);
		view.graphicChanged(this);
	}

	// These should	be overridden if the graphic needs to do any setup
	// before or cleanup after manipulation...
	public void prepareForManipulation(MouseEvent event,SimpleDrawView view) {
		manipulatingGraphic = true;
	}

	public void finishManipulation(MouseEvent event,SimpleDrawView view) {
		manipulatingGraphic = false;
	}

	// This	is called whenever the graphic should display itself...
	// This	should be overridden to	draw the graphic...
	public void drawGraphic(Graphics g) {
	}

	// Draws the receiver's	handles...
	// Subclasses should override handles()	to
	// return a Vector of their handles.
	public void drawHandles(Graphics g) {
		Vector handles = handles();

		for(int	i=0,count=handles.count();i<count;i++) {
			Point p	= ((Point)handles.elementAt(i));
			Rect handleRect	= handleRectForPoint(p);

			// Draw	the highlight...
			g.setColor(Color.white);
			handleRect.x -=	1;
			handleRect.y -=	1;
			g.fillRect(handleRect);

			// Draw	the shadow...
			g.setColor(Color.darkGray);
			handleRect.x +=	2;
			handleRect.y +=	2;
			g.fillRect(handleRect);

			// Draw	the surface...
			g.setColor(Color.lightGray);
			handleRect.x -=	1;
			handleRect.y -=	1;
			g.fillRect(handleRect);
		}
	}

	// Returns a Vector of the graphic's handles (as Points)...
	// This	should be overridden by	subclasses to return handles specific
	// to that kind	of graphic.
	public Vector handles()	{
		Vector handles = new Vector();
		return handles;
	}

	public int anchorForHitHandle(MouseEvent event,SimpleDrawView view) {
		return NONE;
	}

	// Returns true	if one the graphic's handles was hit...
	public boolean hitHandle(MouseEvent event) {
		Vector handles = handles();

		for (int i=0,count=handles.count();i<count;i++)	{
			if (hitHandle(event,((Point)handles.elementAt(i))))
				return true;
		}

		return false;
	}

	// Returns true	if the graphic's handle	at Point p was hit...
	public boolean hitHandle(MouseEvent event,Point	p) {
		Rect handleRect	= handleRectForPoint(p);

		return handleRect.contains(event.x,event.y);
	}

	// Returns a Rect for the handle at Point p...
	public Rect handleRectForPoint(Point p)	{
		return new Rect(p.x-2,p.y-2,5,5);
	}

	// Called when user is attempting to drop a stroke color on the
	// current graphic.  Subclasses	can override to	stop a stroke color
	// from	being dropped on themselves...
	public boolean acceptsDraggedStrokeColor() {
		return true;
	}

	// Called when user is attempting to drop a fill color on the
	// current graphic.  Subclasses	can override to	stop a fill color
	// from	being dropped on themselves...
	public boolean acceptsDraggedFillColor() {
		return true;
	}

	// Used	for hit	detection.  Subclasses should override this
	// method to do	hit detection appropriate for them...
	public boolean intersectsRect(Rect aRect) {
		return false;
	}

	public void moveBy(int deltaX, int deltaY) {
	}

	// Return true if the graphic is currently being manipulated...
	public boolean isBeingManipulated() {
		return manipulatingGraphic;
	}

	// Return the type of events the graphic would like to receive...
	public boolean acceptsEvent(Event event) {
		return (event.type() ==	MouseEvent.MOUSE_DRAGGED |
			event.type() ==	MouseEvent.MOUSE_UP);
	}

	// Accessors

	public void setStrokeColor(Color newColor) {
		strokeColor = newColor;
	}

	public Color strokeColor() {
		return strokeColor;
	}

	public void setFillColor(Color newColor) {
		fillColor = newColor;
	}

	public Color fillColor() {
		return fillColor;
	}

	public void setFilled(boolean flag) {
		filled = flag;
	}

	public boolean isFilled() {
		return filled;
	}

	public void setSelected(boolean	flag) {
		selected = flag;
	}

	public boolean isSelected() {
		return selected;
	}

	public void setID(String ID) {
		this.ID = ID;
	}

	public String id() {
		return ID;
	}
}


