
//
//	Line.java
//
//	Copyright 1996,	Black Diamond Software.	 All rights reserved.
//
//	Written	by: Jason Beaver and Tyler Gingrich
//

package	vt.cs.virtSchool.labapplets.WhiteBoard;

import netscape.application.*;
import netscape.util.*;

public class Line extends Graphic {
	Point start,end;

	// Anchor points...
	private	static final int START_POINT = 0;
	private	static final int END_POINT = 1;

	public Line() {
		super();
	}

	public Line(Point startPoint,Point endPoint) {
		super();

		start =	startPoint;
		end = endPoint;
	}

	// Called to set the receiver up for creating...
	public void createGraphic(MouseEvent event,SimpleDrawView view)	{
		super.createGraphic(event,view);

		start =	new Point(event.x,event.y);
		end = new Point(event.x,event.y);
		anchor = START_POINT;
	}

	// This	is called repeatedly as	the user manipulates the graphic...
	public void updateGraphic(MouseEvent event,SimpleDrawView view)	{
		super.updateGraphic(event,view);

		if (view.currentOperation() == SimpleDrawView.CREATING ||
			view.currentOperation()	== SimpleDrawView.RESIZING) {
			switch (anchor)	{
				case START_POINT:
					end = new Point(event.x,event.y);
					break;
				case END_POINT:
					start =	new Point(event.x,event.y);
					break;
			}
		}
		view.graphicChanged(this);
	}

	// This	is called whenever the graphic should display itself...
	public void drawGraphic(Graphics g) {
		super.drawGraphic(g);
		g.setColor(strokeColor());
		g.drawLine(start.x,start.y,end.x,end.y);
	}

	// Returns a Vector of the graphic's handles (as Points)...
	public Vector handles()	{
		Vector handles = new Vector();

		handles.addElement(start);
		handles.addElement(end);
		return handles;
	}

	// Returns the anchor corresponding to the hit handle...
	public int anchorForHitHandle(MouseEvent event,SimpleDrawView view) {
		Vector handles = handles();

		for (int i=0,count=handles.count();i<count;i++)	{
			Point handlePoint = ((Point)handles.elementAt(i));

			if (hitHandle(event,handlePoint)) {
				switch (i) {
					case 0:	return END_POINT;
					case 1:	return START_POINT;
				}
			}
		}
		return super.anchorForHitHandle(event,view);
	}

	// Overridden to stop lines from accepting dragged fill	colors...
	public boolean acceptsDraggedFillColor() {
		return false;
	}

	// Used	for hit	detection.  This algorithm is crude.  It simply	uses the
	// line's bounding rectangle...
	public boolean intersectsRect(Rect aRect) {
		int minX = (start.x<end.x)?start.x:end.x;
		int minY = (start.y<end.y)?start.y:end.y;
		int width = Math.abs(start.x-end.x);
		int height = Math.abs(start.y-end.y);

		if(width <1) width = 1;
		if(height <1) height = 1;

		if (aRect.intersects(new Rect(minX,minY,width,height)))	{
			return true;
		}

		return false;
	}

	public void moveBy(int deltaX, int deltaY) {
		start.x	+= deltaX;
		start.y	+= deltaY;
		end.x += deltaX;
		end.y += deltaY;
	}
}
