/*
 *
 * Written By:   Philip Isenhour
 * Date:         October 1996
 * Description:  Representation of an available or instantiated activity.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.4 1997/01/07 20:52:58 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
package vt.cs.virtSchool.server;

import java.util.*;
import java.net.*;
import java.io.*;
import netscape.application.*;

/**
 * An <CODE>Activity</CODE> represents an available or instantiated
 * activity applet. This class needs to be cleaned up:
 * <UL>
 * <LI>A better way of describing if and how the activity uses data
 * needs to be derived.
 * </UL>
 *
 * @version $Id Exp $
 */
public class Activity implements Serializable {
  public static final int	NO_IO					= 0;
  public static final int	READS_DATA				= 1;
  public static final int	WRITES_DATA				= 2;
  public static final int	READS_AND_WRITES_DATA	= 3;

  public	int 		x;
  public	int 		y;
  public	int 		width;
  public	int 		height;
  public	int    		type;	
  public	String 		instantiatedBy 	= null;
  public	String 		executedBy 		= null;
  public	String 		name			= null;
  private	String		ID				= null;
  private	String		DatabaseID		= null;
  public	String		appletName		= null;
  public	String		url				= null;
  public	String 		iconURL			= null;
  public	Hashtable	params			= null;

  public Activity(Activity act, String user) {
    x = act.x;
    y = act.y;
    width = act.width;
    height = act.height;
    type = act.type;	
    instantiatedBy = user;
    if (act.executedBy != null)
      executedBy = new String(act.executedBy);
    name = new String(act.name);
    ID = new String(name + "." + Integer.toString(type) + "." +
		    instantiatedBy + "." + System.currentTimeMillis());
    debug("ID = "+ID);

    DatabaseID = new String(name + " (" + instantiatedBy + " at " +
			    new Date() + ")");

    appletName = act.appletName;
    url	= act.url;
    iconURL	= act.iconURL;
    params = act.params;
  }


  public Activity(int x, int y, int width, int height, String appletName, 
		  String iconURL, String instantiatedBy, String name, int type) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.name = name;
    this.type = type;
    this.appletName = appletName;
    this.params = new Hashtable();
    ID = new String(name + "." + Integer.toString(type) + "." +
		    instantiatedBy + "." + System.currentTimeMillis());

    DatabaseID = new String(name + " (" + instantiatedBy + " at " +
			    new Date() + ")");

    try {
      url = Configuration.AppletLauncherScript +
	"?title=\"" + DatabaseID + "\":::" +
	"codebase=\"" + Configuration.CodeBase + "\":::" +
	"code=\"" + Configuration.LabAppletsPackage + "." +
	appletName + "." + appletName + ".class\"";

      /*
       * Add an ApplicationClass parm for Netscape IFC-based
       * applets. Other applets can just ignore this.
       */
      addURLParameter("ApplicationClass",
		      "vt.cs.virtSchool.labapplets." + appletName +
		      "." + appletName + "Applet");

      this.iconURL = iconURL;
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void setSessionURLbase (String sessionURLbase) {
    addURLParameter("SessionURLbase", sessionURLbase);
    addURLParameter("SessionURL", sessionURLbase + ID);
    params.put("ControlChannel", "ControlChannel");
  }

  /**
    setSessionURLbase should be called before this is called.
    Otherwise, this will return null.
   */
  public String getSessionURL() {
    return (String)params.get("SessionURL");
  }

  /**
    setSessionURLbase should be called before this is called.
    Otherwise, this will return null.
   */
  public String getControlChanneName() {
    return (String)params.get("ControlChannel");
  }

  public void addURLParameter(String parm, String value) {
    url = url.concat(":::" + parm + "=\"" + value + "\"");
    params.put(parm, value);
  }

  public Bitmap iconBitmap() {
    try {
      return Bitmap.bitmapFromURL(new URL(iconURL));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

  public String id() {
    return ID;
  }

  public String databaseId() {
    return DatabaseID;
  }

  public boolean generatesData() {
    return (type == WRITES_DATA) || (type == READS_AND_WRITES_DATA);
  }  

  public boolean readsData() {
    debug("readsData() Activity: "+this+" = "+((type == READS_DATA) || (type == READS_AND_WRITES_DATA)));
    return (type == READS_DATA) || (type == READS_AND_WRITES_DATA);
  }


  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }

}



