/*
 *
 * Written By:   Philip Isenhour
 * Date:         October 1996
 * Description:  Interface to (server-side) room data.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.4 1997/01/07 20:52:58 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.server;

import java.rmi.*;
import java.util.*;

/**
 * Interface to (server-side) room data.
 * 
 * @see vt.cs.virtSchool.server.RoomDataImpl
 * @version $Id Exp $
 */
public interface RoomData extends Remote {
	public static final String  ROOMTAG 			= "room";

	public static final String	TABLE_CHANNEL 		= "table_events";
	public static final String	SHELF_CHANNEL 		= "shelf_events";

	public static final String	CHAT_CHANNEL 		= "chat_data";
	public static final String	WHITEBOARD_CHANNEL	= "whiteboard_data";

	public String getRoomName() throws RemoteException;
	public String getDescriptionURL() throws RemoteException;
	public String getSessionURL() throws RemoteException;

  /**
   * Returns the (JSDA) base URL (sessionURL - name); 
   */
  public String getSessionURLbase() throws RemoteException;

       /**
	 * Returns a (hopefully) unique mangled version of the name. (Currently
	 * it just appends a timestamp.) This was added as a work-around for
	 * a JSDA glitch which (I think) has been fixed.
	 */
	public String getUniqueClientName(String name) throws RemoteException;

	/**
 	 * Returns the <I>root</I> of the names of all database sets used
	 * by activities in the system (it's a little mis-named).
	 */
	public String getDatabaseSetName() throws RemoteException;

  /**
   * Opens and holds a session based on sessionURL
   */
  public void holdSession(String sessionURL) throws RemoteException;
  public void closeSession(String sessionURL) throws RemoteException;

  /**
   * Opens and holds a channel in a given session. The session must exist (see holdSession).
   * This method is not used in the virtSchool -Bo 5/15/97
   */
  public void holdChannel(String sessionURL, String channelName) throws RemoteException;
  public void closeChannel(String sessionURL, String channelName) throws RemoteException;



	public Hashtable getShelfActivities() throws RemoteException;
	public Hashtable getTableActivities() throws RemoteException;

	/**
	 * <CODE>permissionToLaunchActivity</CODE> is not currently used.
	 * This functionality (if still desired) should probably be replaced
	 * with a Token-based implementation.
	 */
	public boolean permissionToLaunchActivity(Activity act, String user)
		throws RemoteException;

	/**
	 * <CODE>activityClosed</CODE> is not currently used.
	 * This functionality (if still desired) should probably be replaced
	 * with a Token-based implementation.
	 */
	public void activityClosed(Activity act, String user) 
		throws RemoteException;
}

