/*
 *
 * Written By:   James "Bo" Begole
 * Date:         1 July 1996
 * Description:  Remote Object for Reaction Timer Data
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/util/DataSetsImpl.java,v 1.3 1996/09/17 06:38:40 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */


package vt.cs.virtSchool.util;

import vt.cs.virtSchool.objectManager.*;
import vt.cs.virtSchool.util.*;

import java.util.*;

import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.server.UnicastRemoteServer;
import java.rmi.server.StubSecurityManager;


public class DataSetsImpl 
  extends UnicastRemoteServer
  implements DataSets {

  private final boolean debug = false;

  private String name = null;
  private Hashtable dataSets = new Hashtable();
  private boolean newDataSet = false;

  public DataSetsImpl () throws RemoteException {
  }

  public DataSetsImpl (String name) throws RemoteException {
    super();
    this.name = name;
  }

	
  // Producer Methods
  public void addDataSet(String dataSetName, DataSet dataSet) throws java.rmi.RemoteException {
      dataSets.put(dataSetName, dataSet);
      newDataSet = true;
  }


  public void removeDataSet(String dataSetName) throws java.rmi.RemoteException {
    Object returnValue = dataSets.remove(dataSetName);
    if (returnValue == null)
      System.out.println("DataSetsImpl.removeDataSet(): DataSet not found in table: "+dataSetName);
      newDataSet = true;
  }

  // Consumer Methods
  public Vector getPoints() throws java.rmi.RemoteException {
    Vector allPoints = new Vector();
    try {
      for(Enumeration e = dataSets.elements(); e.hasMoreElements();) {
	DataSet dataSet = (DataSet)e.nextElement();
	Vector somePoints = dataSet.getPoints();
	for (Enumeration e2 = somePoints.elements(); e2.hasMoreElements();){
	  Long point = (Long)e2.nextElement();
	  allPoints.addElement(point);
	}
      }
    } catch (RemoteException re) {
      System.out.println("DataSetsImpl.getPoints(): RemoteException: "+re);
    }
    return allPoints;
  }

  // PLI ----------------------------------------------------------------------
  public Vector get(String key) throws java.rmi.RemoteException {
    Vector allPoints = new Vector();
    try {
      for(Enumeration e = dataSets.elements(); e.hasMoreElements();) {
	DataSet dataSet = (DataSet)e.nextElement();
	Vector somePoints = dataSet.get(key);
	if (somePoints != null) {
	  for (Enumeration e2 = somePoints.elements(); e2.hasMoreElements();){
	    Object point = e2.nextElement();
	    allPoints.addElement(point);
	  }
	}
      }
    } catch (RemoteException re) {
      System.out.println("DataSetsImpl.get(): RemoteException: "+re);
    }
    return allPoints;
  }

  public Vector getKeys(String typeName) throws java.rmi.RemoteException {
    Vector allKeys = new Vector();
    try {
      for(Enumeration e = dataSets.elements(); e.hasMoreElements();) {
	DataSet dataSet = (DataSet)e.nextElement();
	Vector keys = dataSet.getKeys(typeName);
	if (keys != null) {
	  for (Enumeration e2 = keys.elements(); e2.hasMoreElements();) {
	    Object key = e2.nextElement();
	    allKeys.addElement(key);
	  }
	}
      }
    } catch (RemoteException re) {
      System.out.println("DataSetsImpl.getKeys(): RemoteException: "+re);
    }
    return allKeys;
  }

  // PLI ----------------------------------------------------------------------

  public boolean newData() throws java.rmi.RemoteException {
    if (newDataSet) {
      newDataSet = false;
      return true;
    }

    try {
	for(Enumeration e = dataSets.elements(); e.hasMoreElements();) {
	  DataSet dataSet = (DataSet)e.nextElement();
	  if (dataSet.newData())
	    return true;
	}
    } catch (RemoteException re) {
      if (debug) System.out.println("DataSetsImpl.newData() "+re);
    }
    return false;
  }

  public long maxValue() throws java.rmi.RemoteException {
    long maxPointValue = Long.MIN_VALUE;
    for(Enumeration e = dataSets.elements(); e.hasMoreElements();) {
      DataSet dataSet = (DataSet)e.nextElement();
      try {
	if (dataSet.maxValue() > maxPointValue)
	  maxPointValue = dataSet.maxValue();
      } catch (RemoteException re) {
	System.out.println("DataSetsImpl.maxValue(): RemoteException: "+re);
      }
    }
    return maxPointValue;
  }

  public long minValue() throws java.rmi.RemoteException {
    long minPointValue = Long.MAX_VALUE;
    for(Enumeration e = dataSets.elements(); e.hasMoreElements();) {
      DataSet dataSet = (DataSet)e.nextElement();
      try {
	if (dataSet.minValue() < minPointValue)
	  minPointValue = dataSet.minValue();
      } catch (RemoteException re) {
	System.out.println("DataSetsImpl.minValue(): RemoteException: "+re);
      }
    }
    return minPointValue;
  }

  public void setName (String name) throws RemoteException {
    this.name = name;
  }
}
