/*
 *
 * Written By:   James "Bo" Begole
 * Date:         7 July 1996
 * Description:  Speedy Remote Vector implementation
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/util/FastRemoteVector.java,v 1.1.1.1 1996/09/17 05:23:18 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.util;

//import vt.cs.ro.remote.*;
import vt.cs.virtSchool.objectManager.*;
//import vt.cs.ro.util.*;

import java.rmi.Remote;
import java.rmi.RemoteException;

import java.util.*;

public class FastRemoteVector {
  private final boolean debug = false;

  private ObjectManager objectManager = null;
  private RemoteVector remoteVector = null;
  private Vector localVector = null;
  private String name = null;
  private long ID = 0;


  public FastRemoteVector(ObjectManager objectManager, String name) {
    this.objectManager = objectManager;
    this.name = name;

    remoteVector = getRemoteVector(name);
    try {
      ID = remoteVector.register();
      localVector = remoteVector.getVector(ID);
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.Construvtor: Remote Exception: "+re);
    }  
  }

  private RemoteVector getRemoteVector(String name) {
      String url=null;
      try {
	url = objectManager.retrieveNamedObject(name);
	if (debug) System.out.println(name+" URL = "+url);
      } catch (java.rmi.RemoteException re) {
	System.out.println("Remote Exception during retrieveNamedObject: "+re);
      }  

      java.rmi.Remote obj = RemoteObject.getRemoteObject(url);
      if (obj instanceof RemoteVector)
	return (RemoteVector)obj;
      else
	return null;
  }

  public void addElement(Object obj) {
    try {
      remoteVector.addElement(obj);
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.addElement(): Remote Exception: "+re);
    }      
  }

  public void removeElementAt(int index) {
    try {
       remoteVector.removeElementAt(index);
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.removeElement(): Remote Exception: "+re);
    }     
  }

  public boolean removeElement(Object obj) {
    boolean returnValue = false;
    try {
      returnValue = remoteVector.removeElement(obj);
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.removeElement(): Remote Exception: "+re);
    }     
    return returnValue;
  }

  public Enumeration elements() {
    checkRemote();
    return localVector.elements();
  }

  public Object lastElement() {
    Object returnValue = null;
    try {
      returnValue = remoteVector.lastElement();
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.size(): Remote Exception: "+re);
    }     
    return returnValue;
  }

  public int size() {
    int returnValue = 0;
    try {
      returnValue = remoteVector.size();
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.size(): Remote Exception: "+re);
    }     
     return returnValue;
 }


  private void checkRemote() {
    try {
      if (remoteVector.newData(ID)) {
	localVector = remoteVector.getVector(ID);
      }
    } catch (java.rmi.RemoteException re) {
      System.out.println("FastRemoteVector.removeElement(): Remote Exception: "+re);
    }     

  }

}
