/*
 *
 * Written By:   James "Bo" Begole
 * Date:         2 July 1996
 * Description:  Remote Object Server implementation
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/util/Remote.java,v 1.1.1.1 1996/09/17 05:23:18 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.util;

import vt.cs.virtSchool.objectManager.ObjectManager;
//import vt.cs.ro.remote.*;

import java.rmi.Naming;
import java.applet.Applet;
import java.net.URL;
import java.net.InetAddress;
import java.util.Vector;

public class Remote {
  private static final boolean debug = true;
  private static final int MAXDATASETS = 10;


  public static ObjectManager getObjectManager(Applet applet) {

    String url = applet.getParameter("ObjectManager");
    if (url == null) {
      try {
	SecurityManager mgr = System.getSecurityManager();
	Object context;
	String host;
	if (mgr != null &&
	    (context = mgr.getSecurityContext()) != null &&
	    context instanceof URL) {
	  host = ((URL)context).getHost();
	} else {
	  String prot = applet.getCodeBase().getProtocol();
	  if (prot.equals("file"))
	    host = InetAddress.getLocalHost().getHostName();
	  else
	    host = applet.getCodeBase().getHost();
	}
	url = "//" + host + "/ObjectManager";
      } catch (java.net.UnknownHostException ex) {
	applet.showStatus("Default host is unknown.");
      }
    }

    java.rmi.Remote obj = getRemoteObject(url);
    if (obj instanceof ObjectManager)
      return (ObjectManager)obj;
    else
      return null;
  }

  public static Vector getDataSets(Applet applet, ObjectManager objectManager) {
    Vector dataSets = new Vector();
      String dataObjPrefix = "DataSet";
      for (int i = 0; i<MAXDATASETS; i++) {
	String parmString = new String(dataObjPrefix+i);
	if (debug) System.out.println("Param Name = "+parmString);
	String dataSetName = applet.getParameter(parmString);
	if (dataSetName == null)
	  break;
	else {
	  String url=null;
	  try {
	    url = objectManager.retrieveNamedObject(dataSetName);
	    if (debug) System.out.println("DataSet URL = "+url);
	  } catch (java.rmi.RemoteException re) {
	    System.out.println("Remote Exception during retrieveNamedObject: "+re);
	  }
	  DataSet dataSet = getDataSet(url);
	  if (dataSet != null)
	    dataSets.addElement(dataSet);
	  else
	    if (debug) System.out.println("DataSet not found: "+url);
	}
      }
      return dataSets;
    }

  private static DataSet getDataSet(String where){
    java.rmi.Remote obj = getRemoteObject(where);
    if (obj instanceof DataSet)
      return (DataSet)obj;
    else
      return null;
  }


  private static java.rmi.Remote getRemoteObject(String where) {
    java.rmi.Remote obj = null;
    try {
      obj = Naming.lookup(where);
      if (obj != null) {
	System.out.println("Found " + where + ".");
      } else {
	System.out.println("Named object does not exist");
      }
    } catch (java.rmi.NotBoundException ex) {
      System.out.println("Name not bound.");
    } catch (java.net.UnknownHostException ex) {
      System.out.println("Host unknown in where " + where);
    } catch (java.rmi.RemoteException ex) {
      ex.printStackTrace();
      System.out.println("RemoteException during lookup.");
    } catch (java.net.MalformedURLException ex) {
      System.out.println("Malformed URL " + where);
    }
    return obj;
  }
}
