/*
 *
 * Written By:   James "Bo" Begole
 * Date:         6 July 1996
 * Description:  Remote Vector Implementation
 *
 * Usage Notes:
 *
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/util/RemoteVectorImpl.java,v 1.3 1996/09/17 06:38:43 isenhour Exp $
 *
 * Copyright (c) 1996 James "Bo" Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */


package vt.cs.virtSchool.util;

import vt.cs.virtSchool.util.*;

import java.util.*;

import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.server.UnicastRemoteServer;
import java.rmi.server.StubSecurityManager;


public class RemoteVectorImpl 
  extends UnicastRemoteServer
  implements RemoteVector {

  private final boolean debug = false;

  private Vector vector = new Vector();
  private Vector readers = new Vector();

  public RemoteVectorImpl () throws RemoteException {
  }

  public void addElement(Object obj) throws java.rmi.RemoteException {
    vector.addElement(obj);
    for (Enumeration e = readers.elements(); e.hasMoreElements();) {
      Reader r = (Reader)e.nextElement();
      r.setTrue();
    }
  }

  public boolean removeElement(Object obj) throws java.rmi.RemoteException {
    boolean returnValue =  vector.removeElement(obj);
    for (Enumeration e = readers.elements(); e.hasMoreElements();) {
      Reader r = (Reader)e.nextElement();
      r.setTrue();
    }
    return returnValue;
  }

  public void removeElementAt(int index) throws java.rmi.RemoteException {
    try {
    vector.removeElementAt(index);
    } catch (ArrayIndexOutOfBoundsException ex) {
	  System.out.println("RemoteVectorImpl.removeElementAt(): Array index out of bounds.  Index = "+
			     index+" . Vector size = "+vector.size());
    }
    for (Enumeration e = readers.elements(); e.hasMoreElements();) {
      Reader r = (Reader)e.nextElement();
      r.setTrue();
    }
  }

  public Enumeration elements() throws java.rmi.RemoteException {
    return vector.elements();
  }

  public Object lastElement() throws java.rmi.RemoteException {
    return vector.lastElement();
  }

  public int size() throws java.rmi.RemoteException {
    return vector.size();
  }

  public long register() throws java.rmi.RemoteException {
    long ID = System.currentTimeMillis();
    Reader r = new Reader(ID, true);
    readers.addElement(r);
    return ID;
  }

  public void unregister(long ID) throws java.rmi.RemoteException {
    int j = 0;
    for (Enumeration e = readers.elements(); e.hasMoreElements(); j++) {
      Reader r = (Reader)e.nextElement();
      if (r.ID() == ID) {
	try {
	  readers.removeElementAt(j);
	} catch (ArrayIndexOutOfBoundsException ex) {
	  System.out.println("RemoteVectorImpl.unregister(): Array index out of bounds.  Index = "+
			     j+" . Vector size = "+readers.size());
	}
	break;
      }
    }
  }

  public Vector getVector(long ID) throws java.rmi.RemoteException {
    for (Enumeration e = readers.elements(); e.hasMoreElements();) {
      Reader r = (Reader)e.nextElement();
      if (r.ID() == ID) {
	r.setFalse();
	break;
      }
    }
    return vector;
  }

  public boolean newData(long ID) throws java.rmi.RemoteException {
    for (Enumeration e = readers.elements(); e.hasMoreElements();) {
      Reader r = (Reader)e.nextElement();
      if (r.ID() == ID) {
	return r.newData();
      }
    }
    return true;
  }


}
