/*
 * Written By:   James "Bo" Begole
 * Date:         5 June 1996
 * Description:  Panel to list sessions in Virtual School
 *
 * Usage Notes:
 *
 * 
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/worldClient/SessionsPanel.java,v 1.1.1.1 1996/09/17 05:23:17 isenhour Exp $
 * Copyright (c) 1996 James "Bo" Begole, Craig Struble
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.worldClient; // virtSchool.util;

import java.awt.*;
import java.util.*;

import vt.cs.widgets.*;
import vt.cs.virtSchool.worldClient.WorldClient;
import vt.cs.virtSchool.worldClient.IrcChan;

import como.awt.SmartFrame;

/**
  SessionsPanel is a Panel component that contains a list of the currently active 
  Channels that begin with a specified prefix string.
  */

public class SessionsPanel extends Panel implements MultiColumnCallback {
  private final boolean debug = true;
  private final static int defaultListRows = 5;
  
  private int listRows;   // the number of rows to display in the list
  private SessionAccessor sessionAccessor;  // the object that knows how to get to the channels
  private String prefix;  // The prefix of channels to list
  private UsersPanel usersPanel;  // A users panel that should be updated when a channel is selected

  private Button gotoButton;  // a button that causes the user to join the selected channel when pressed
  private MultiColumn list;  // the list component
	
  public SessionsPanel (SessionAccessor sessionAccessor, String title, String prefix, int listRows) {
    this(sessionAccessor, title, prefix, null, listRows);
    if (debug)
      System.out.println("SessionsPanel: Constructor 1:");
  }
	
  public SessionsPanel (SessionAccessor sessionAccessor, String title, String prefix) {
    this(sessionAccessor, title, prefix, null, defaultListRows);
    if (debug)
      System.out.println("SessionsPanel: Constructor 2:");
  }
  
  public SessionsPanel (SessionAccessor sessionAccessor, 
			String title, 
			String prefix, 
			UsersPanel usersPanel,
			int listRows) {
    if (debug)
      System.out.println("SessionsPanel: Constructor 3:");
    this.listRows = listRows;
    this.usersPanel = usersPanel;
    this.sessionAccessor = sessionAccessor;
    this.prefix = new String(prefix);
		
    String titles[] = new String[2];
    titles[0] = title;
    titles[1] = "topic";
    list = new MultiColumn(titles, listRows, this);

		
    // Set up places Button Panel
    gotoButton = new Button("Goto");
    gotoButton.disable();
    Panel buttons = new Panel();
    buttons.add(gotoButton);
    buttons.add(new Button("Update List"));
		
    // Set up places panel
    setLayout(new BorderLayout(0, 0));
    add("North", new Label(title));
    add("Center", list);
    add("South", buttons);
    update();  
   
    if (debug)
      System.out.println("SessionsPanel: initialize(): end");
  }
	
  IrcChan[] channels;
  Hashtable channelHT;
	
  public synchronized void update() {
    if (debug)
      System.out.println("SessionsPanel: update:");
	  
    String oldSelection[] = null;
    // if something is selected, save it
    if (list.selected() != -1) {	// something is selected
      oldSelection = list.getItem(list.selected());
    }
	  	
    // Rebuild the whole list
    list.clear();

    channels = sessionAccessor.getChannels("*");
    channelHT = new Hashtable();
			
    String s = null;
    String blanks = "                         ";
    for (int i = 0;i<channels.length;i++) {
      IrcChan c = channels[i];
      s = c.commletname;
      if (s.startsWith(prefix)) {
	// cut the prefix off the string
	s = s.substring(prefix.length());
	// see if the string contains a second delimiter
	StringTokenizer tokenizer = new StringTokenizer(s, WorldClient.SESSION_DELIMITER);
	if (tokenizer.countTokens() == 1) {
	  // this is a place of the prefix type,
	  // so add it to the list
	  String tempStringArray[] = new String[2];
	  tempStringArray[0]=s;
	  tempStringArray[1]=c.topic;
	  list.addItem(tempStringArray);
	  channelHT.put(s,new Integer(i));
	}
	// otherwise ignore this channel, it is a subsession of this session
      }
    }
		   
    // if there was an item selected previously, reselect it, if it still is in the list
    if (oldSelection != null) {
      for (int j = 0; j < list.getRows(); j++) {
	if (oldSelection.equals(list.getItem(j))) {
	  list.select(j);
	  break;
	}
      }
      if (usersPanel != null)
	updateUserList();
				
      gotoButton.enable();
    } else {
      gotoButton.disable();
    }
  }

	
  private synchronized void updateUserList() {
    if (usersPanel == null)	// this is not linked to a usersPanel object
      return;			// so do nothing and return
			
    String searchPattern = "";	

    if (list.selected() != -1) {
      searchPattern = WorldClient.WORLD_CHANNEL_PREFIX+prefix+list.getItem(list.selected())[0];
    } else {	// no room selected
      // open a smart frame with the message to select a place
      new SmartFrame( "Select a place to list the users there." );
      return;
    }
			
    usersPanel.update(searchPattern);		
  }


  public boolean action(Event evt, Object what) {
		
    if (gotoButton.getLabel().equals(what)){
      sessionAccessor.joinSession( prefix+list.getItem(list.selected())[0] );
      return true;
    } else if ("Update List".equals(what)) {
      update();
      return true;
    }

    return false;
  }
  

  public void singleClick(MultiColumn mcList, int row) {
    if (debug)
      System.out.println("singleClick: List= "+ mcList+ "| row= "+row);
    updateUserList();
    gotoButton.enable();
  }

  public void doubleClick(MultiColumn mcList, int row) {
    if (debug){
      System.out.println("doubleClick: List= "+ mcList+ "| row= "+row);
      System.out.println("Selected "+list.getItem(row)[0]+"  "+list.getItem(list.selected())[0]);
    }
    sessionAccessor.joinSession( prefix+list.getItem(list.selected())[0] );
  }
  


  public void paint(Graphics g) {
    super.paint(g);
    update();
  }
		
}

	
