/*
 * Written By:   James "Bo" Begole
 * Date:         17 June 1996
 * Description:  Panel to list people in Virtual School
 *
 * Usage Notes:
 *
 * 
 */
 
/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/worldClient/UsersPanel.java,v 1.1.1.1 1996/09/17 05:23:17 isenhour Exp $
 * Modified for use by the NIE project.
 * Copyright (c) 1996 James "Bo" Begole, Craig Struble
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package vt.cs.virtSchool.worldClient; // virtSchool.util;

import java.awt.*;
import java.util.*;

//import como.sys.*;
import como.sys.User;
import como.awt.SmartFrame;

public class UsersPanel extends Panel {
	private final boolean debug = false;
	
	private int listRows = 5;
	private SessionAccessor sessionAccessor = null;
	private List list = null;
	private Button talkButton = null;
	
	
	public UsersPanel(SessionAccessor sessionAccessor, String title, int listRows) {
		this.listRows = listRows;
		initialize(sessionAccessor, title);
	}
	
	public UsersPanel(SessionAccessor sessionAccessor, String title) {
		initialize(sessionAccessor, title);
	}


 	private void initialize(SessionAccessor sessionAccessor, String title) {
		this.sessionAccessor = sessionAccessor;
		
		list = new List(listRows, true);
		
		// Set up Button Panel
		talkButton = new Button("Talk");
		talkButton.disable();
		Panel buttons = new Panel();
		buttons.add(talkButton);

		// Set up list
		setLayout(new BorderLayout());
		add("North", new Label("Who is there:"));
		add("Center", list);
		add("South", buttons);
	}
	
	
	
	private User[] users;
	private Hashtable userHT;
	private String lastSearchString;

	public synchronized void update(String searchPattern) {
		lastSearchString = searchPattern;
		userHT = new Hashtable();
		
		clear();
		talkButton.disable();
		
		boolean comoonly = false;
		
		users = sessionAccessor.getUsers(searchPattern, comoonly);
		
		for (int i = 0;i<users.length;i++) {
			User u = users[i];
			StringBuffer s = new StringBuffer();

			if (u.containsKey(u.NAME)) s.append(u.get(u.NAME).toString());
			// if (u.containsKey(u.NICK)) s.append(" ("+u.get(u.NICK).toString()+")");

			if (u.containsKey(u.COMMENT)) s.append(" ("+u.get(u.COMMENT).toString()+")"); 
			String tempString = s.toString();
			if (!tempString.startsWith("NIE World Keeper"))
				list.addItem(tempString);
			userHT.put(s.toString(),new Integer(i));
		}
		
	}
	
	public synchronized void update() {
		update(lastSearchString);
	}
	
	public synchronized void clear() {
		list.delItems(0, list.countItems()-1);
	}


	public boolean handleEvent(Event e) {
		if (e.target instanceof List) {
			switch (e.id) {
				case Event.ACTION_EVENT:
					break;
				case Event.LIST_SELECT:
					int sIndex = ((Integer)e.arg).intValue();
					System.out.println("Select event occurred on item #"
						+ sIndex + " (\""
						+ list.getItem(sIndex)  + "\") " );
					talkButton.enable();
					break;
				case Event.LIST_DESELECT:
					int dIndex = ((Integer)e.arg).intValue();
					System.out.println("Deselect event occurred on item #"
						+ dIndex + " (\""
						+ list.getItem(dIndex)  + "\") " );
					if (list.getSelectedItem() == null) {	// no other users are selected
						talkButton.disable();
					}
			}
		} 
		return super.handleEvent(e);
	}	



	public boolean action(Event evt, Object what) {
		if (debug)
			System.out.println("action:"+evt);
		
		else if ("Talk".equals(what)) {
			// Start a superchat session with the selected users
			
			int[] selints = list.getSelectedIndexes();
			if( selints.length > 0 )	{
				User[] u = new User[selints.length];

				for( int j=0; j < selints.length; j++ ) {
					u[j] = users[selints[j]];
					if (debug)
						System.out.println(" User: "+u[j]);
				}
	
				String commletname = "SuperChat";
				sessionAccessor.startCommunication(
					commletname, u, commletname + " started on " + (new Date()).toString());
			}
			else {
				new SmartFrame( "Select some users with which to talk." );
			}
			return true;
		}

		return false;
	}


	public void paint(Graphics g) {
		super.paint(g);
		update();
	}

}
