/*
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/worldKeeper/worldKeeper.java,v 1.1.1.1 1996/09/17 05:23:17 isenhour Exp $
 *
 * worldKeeper - This keeps the channels for the NIE world open by
 *               reading in a room description file, opening IRC channels
 *               for each room and setting their topics, using an
 *               invisible user. Basically a bot that does nothing except
 *               create the channels.
 *
 * Part of the NIE project source.
 *
 * Copyright (c) 1996 James "Bo" Begole, Craig Struble
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.worldKeeper;

import java.io.*;
import java.net.*;

import vt.cs.virtSchool.worldClient.WorldClient;

public class worldKeeper {
  static int IRC_PORT = 6667;
  static String IRC_SERVER = "simon.cs.vt.edu";
  static String ROOM_PREFIX = WorldClient.WORLD_CHANNEL_PREFIX+WorldClient.LEVEL_1_DELIMITER;
  static String KEEPER_NAME = "NIE World Keeper";
  static String KEEPER_NICK = "NIE_KEEPER";
  static String ROOM_CONF = "rooms.conf";
  static String hostname;

  static DataInputStream in;
  static DataOutputStream out;
  static Socket socket;

  final static int RPL_ENDOFMOTD = 376;

  public static void main(String argv[]) {
    int irc_port = IRC_PORT;
    String irc_server = IRC_SERVER;

    /*
     * Get the IRC server and port if passed in
     */
    if (argv.length >= 1) {
      irc_server = argv[0];
    }

    if (argv.length == 2) {
      irc_port = Integer.parseInt(argv[1]);
    } else if (argv.length > 2) {
      usage();
      System.exit(1);
    }
    
    try {
      hostname = InetAddress.getLocalHost().getHostName();
    } catch (Exception e) {
      System.err.println("Cannot get local host name!");
      System.exit(1);
    }

    connectToServer(irc_server, irc_port);
    createRooms();
    runForever();
  }

  /*
   * Connect to the IRC server
   */
  private static void connectToServer(String host, int port) {
    try {
      socket = new Socket( host, port );
      in  = new DataInputStream( socket.getInputStream() );
      out = new DataOutputStream( socket.getOutputStream() );
    } catch( Exception e ) {
      System.err.println("connectToServer(): " + e );
      return;
    }

    /*
     * Log in as the keeper
     */
    send( "USER "+KEEPER_NICK+" "+host+" "+host+" :"+KEEPER_NAME, true );
    send( "NICK "+KEEPER_NICK, true );

    // Get lines until end of MOTD gotten
    int cmd = 0;
    while (cmd != RPL_ENDOFMOTD) {
      String line = null;
      int space,pos,pos2;
      char ch;

      try {
	line = in.readLine();
      } catch(Exception e) {
	System.err.println("Error reading MOTD: " + e);
	System.exit(1);
      }
      
      space = line.indexOf( ' ' );
      pos = space + 1;
      ch = line.charAt( pos );
      if ( ch >= '0' && ch <= '9' ) {
	pos2 = line.indexOf( ' ', pos);
	cmd = Integer.parseInt(line.substring(pos,pos2));
      }
    }

    // Make Keeper invisible
    send( "MODE " + KEEPER_NICK + " +i", true );
    
    // Log in to more than one channel please
    send( "SET NOVICE OFF", true);
  }

  /*
   * Create the rooms
   */
  private static void createRooms() {
    DataInputStream rooms = null;

    try {
      rooms = new DataInputStream(new FileInputStream(ROOM_CONF));
    } catch(Exception e) {
      System.err.println("Error opening room configuration file: " + e);
      System.exit(2);
    }

    /*
     * Try joining and setting the topic of each room
     */
    try {
      String room;
      String topic;
      while ((room = rooms.readLine()) != null) {
	send("JOIN "+ ROOM_PREFIX + room, true);

	if ((topic = rooms.readLine()) != null) {
	  send("TOPIC "+ ROOM_PREFIX + room + " :" + topic, true);
	}
      }
      rooms.close();
    } catch (Exception e) {
      System.err.println("Error reading configuration file: " + e);
      System.exit(3);
    }
  }

  /*
   * Keep going
   */
  private static void runForever() {
    String line = null;
    char ch;
    
    /*
     * Play the PING PONG game with the server
     */
    for(;;) {
      try {
	line = in.readLine();
      } catch(Exception e) {
	System.err.println("Error reading from server: " + e);
	System.exit(3);
      }
      
      if (line.length() == 0) {
	continue;
      }

      if( line.startsWith( "PING" ) ) {
	  String s = "PONG "+ hostname;
	  send( s.toString(), true );
      }
    }
  }

  /*
   * Print out the usage string
   */
  private static void usage() {
    System.out.println("Usage: worldKeeper [Server [Port]]");
  }

  /*
   * Send strings to the server
   */
  private static synchronized void send(String s) {
    if ( out == null) return;

    try {
      out.writeBytes(s);
    } catch (Exception e) {
      System.err.println("send(): " + e);
    }
  }

  /**
   * Send a String to this channel/IRC-Server.
   * Used to send IRC-Commands.
   */
  private static void send( String s, boolean cr ) {
    if( cr == true )
      send( s+"\n" );
    else
      send( s );
  }
        
}
