/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package vt.cs.widgets;

import java.awt.*;

/**
 * BorderPanel - a 3D look panel.
 *
 * @version 	%I%, %G%
 * @author 	James Begole
 */

public class BorderPanel extends Panel {
  private static final boolean debug = false;
  protected int perspective = RAISED;

  protected int border = 5;	// size of border
  protected Color col1 = null;
  protected Color col2 = null;

  public static final int SUNKEN = 0;
  public static final int RAISED = 1;

  /** Constructs a BorderPanel with the default values 
   */
  public BorderPanel() {
  }
  
  /** Construcs a BorderPanel
   * @param borderWidth    width of border
   */
  public BorderPanel(int borderWidth) {
    border = borderWidth;
  }
  
  /** Construcs a BorderPanel
   * @param borderWidth    width of border
   * @param perspective    RAISED or SUNKEN
   */
  public BorderPanel(int borderWidth, int perspective) {
    border = borderWidth;
    this.perspective = perspective;
  }
  
  /** Construcs a BorderPanel
   * @param borderWidth    width of border
   * @param topLeft   Color of top-left border
   * @param bottomRight   Color of bottom-right border
   */
  public BorderPanel(int borderWidth, Color topLeft, Color bottomRight) {
    border = borderWidth;
    col1 = topLeft; col2 = bottomRight;
  }

  /** Construcs a BorderPanel
   * @param topLeft   Color of top-left border
   * @param bottomRight   Color of bottom-right border
   */
  public BorderPanel(Color topLeft, Color bottomRight) {
    col1 = topLeft; col2 = bottomRight;
  }

  /** Construcs a BorderPanel
   * @param borderWidth    width of border
   * @param perspective    RAISED or SUNKEN
   * @param topLeft   Color of top-left border
   * @param bottomRight   Color of bottom-right border
   */
  public BorderPanel(int borderWidth, int perspective, Color topLeft, Color bottomRight) {
    border = borderWidth;
    col1 = topLeft; col2 = bottomRight;
    this.perspective = perspective;
  }

  public Insets insets() {
    return new Insets(border+2, border+2, border+2, border+2);
  }

  public void paint(Graphics g) {
  	Dimension size = size();
/*
        Shape oldClip = g.getClip();
        g.setClip(border, border, size.width-border,
                size.height-border);
        super.paint(g);
        g.setClip(oldClip);

*/
  	Rectangle oldClip = g.getClipRect();
  	g.clipRect(border, border, size.width-border, 
		size.height-border);
    	super.paint(g);
  	g.clipRect(oldClip.x, oldClip.y, size.width, 
		size.height);
    	
    int w = size.width-1, h = size.height-1;
    
    if (debug) 
      System.out.println("Panel Size= "+size());

    if (col1 == null)
      if (perspective == RAISED)
	g.setColor(getBackground().brighter());
      else
	g.setColor(getBackground().darker());
     else
      g.setColor(col1);
			
    for(int i=0; i<border; i++) {
      g.drawLine(i,i,w-i,i);
      g.drawLine(i,i,i,h-i);
    }
			
    if (col2 == null)
      if (perspective == RAISED)
	g.setColor(getBackground().darker());
      else
	g.setColor(getBackground().brighter());
    else
      g.setColor(col2);
    for(int i=0; i<border; i++) {
      g.drawLine(w-i,h-i, w-i,i);
      g.drawLine(w-i,h-i, i,h-i);
    }
  }

  public void update(Graphics g) {
    paint(g);
  }
}

