/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package vt.cs.widgets;

import java.awt.*;
import java.awt.event.*;

/**
 * RadarPaneClientPanel - a panel used by RadarPane
 *
 * @version 	%I%, %G%
 * @author 	James Begole
 */

class RadarPaneClientPanel extends Panel {
  ClientPanelListener listener = null;

  RadarView radarView = null;

  RadarPaneClientPanel() {
    setLayout(null);
    listener = new ClientPanelListener();
    addContainerListener(listener);
  }

  public void setRadarView(RadarView radarView) {
    this.radarView = radarView;
    listener.setRadarView(radarView);
  }

   /**
     * Adds the specified component to this container at the specified
     * index.  Also notifies the layout manager to add the component to
     * the this container's layout using the specified constraints object.
     * <p>
     * This is overriden for RadarPane so that components are added only
     * to the viewport region of the RadarPane.
     * <p>
     * This is the method to override if you want to track every add
     * request to a container.  An overriding method should usually
     * include a call to super.addImpl(comp, constraints, index).
     * @param comp the component to be added
     * @param constraints an object expressing layout contraints for this
     * component
     * @param index the position in the container's list at which to
     * insert the component.  -1 means insert at the end.(must be <= 0 for
     * RadarPaneClientPanel) 
     * @see #remove
     * @see LayoutManager
     */
  protected void addImpl(Component comp, Object constraints, int index) {
    debug("addImpl Comp="+comp+", constraints="+constraints+", index="+index);
    if (index > 0) {
      throw new IllegalArgumentException("position greater than 0");
    }
    super.addImpl(comp, constraints, index);
  }

  public void update(Graphics g) {
    debug("update");
    paint(g);
  }

  /*
  // debug methods
  public void paint(Graphics g) {
    debug("paint");
    super.paint(g);
  }
  public void print(Graphics g) {
    debug("print");
    super.print(g);
  }
  public void printAll(Graphics g) {
    debug("printAll");
    super.printAll(g);
  }
  */

  Component getComponent() {
    return getComponent(0);
  }


    /** 
     * Validates this Container and all of the components contained
     * within it. 
     * @see #validate
     * @see Component#invalidate
     */
    public void validate() {
      super.validate();
      getParent().validate();
    }


    /** 
     * Reshapes the Component to the specified bounding box.
     * Overriden to notify the radar view that the view port has changed.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the component
     * @param height the height of the component
     * @see #getBounds
     * @see #setLocation
     * @see #setSize
     */
  public void setBounds(int x, int y, int width, int height) {
    debug("setBounds("+x+", "+y+", "+width+", "+height+")");
    super.setBounds(x, y, width, height);
    if (radarView != null) {
      radarView.setClientChanged(true);
      //      radarView.setRadarPaneChange(true);
    }
    debug("bounds="+getBounds());
  }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}


//------------------------------------------------------
class ClientPanelListener implements ContainerListener {

  RadarView radarView;

  ClientPanelListener() {
  }

  public void setRadarView(RadarView radarView) {
    this.radarView = radarView;
  }


  public void componentAdded(ContainerEvent e) {
    debug("componentAdded: "+e);
    radarView.setClientChanged(true);
    radarView.repaint();
  }

  public void componentRemoved(ContainerEvent e) {
    debug("componentRemoved: "+e);
    radarView.setClientChanged(true);
    radarView.repaint();
  }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}
