/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package vt.cs.widgets;

import java.awt.*;
/**
 * ViewWindow - a simple window that displays one component.
 *  ViewWindow calls ViewWindowParent.hideViewWindow() when
 *  a WINDOW_DESTROY event occurs on it, but takes no other 
 * action.
 *
 * @version 	%I%, %G%
 * @author 	James Begole, begolej@vt.edu
 */


public class ViewWindow extends Dialog {
  private ViewWindowParent parent = null;

  public ViewWindow(Frame frame, String title, Component view, ViewWindowParent parent){
    super(frame, title);
    this.parent = parent;
    setLayout(new BorderLayout());
    setView(view);
    resize(getPreferredSize());
  }
  
  public void setView(Component view) {
    add("Center", view);
  }
  
  public boolean handleEvent(Event evt) {
    switch (evt.id) {
    case Event.WINDOW_MOVED:
      // call setSize() to relayout components
      setSize(size());
      break;
    case Event.WINDOW_DESTROY:
    	parent.hideViewWindow();
      break;
    }
    return super.handleEvent(evt);
  }
  
  public void update(Graphics g) {
    debug("update()");
    paint(g);
  }

  /*
    // debug overrides
  public void paint(Graphics g) {
    debug("paint()");
    super.paint(g);
  }

  public void doLayout() {
    debug("doLayout()");
    super.doLayout();
   }

  public void validate() {
    debug("validate()");
    super.validate();
  } 

  public void invalidate() {
    debug("invalidate()");
    super.invalidate();
  }
  public Dimension getMinimumSize() {
    Dimension dim = super.getMinimumSize();
    debug("getMinimumSize() returning "+dim);
    return dim;
  }

  public Dimension getPreferredSize() {
    Dimension dim = super.getPreferredSize();
    debug("getPreferedSize() returning "+dim);
    return dim;
   }

   */

 private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}


